<?php
require_once('Admin.inc.php');

final class AdminT3 extends Admin {

	/**
	 * Execute admin tab 3
	 *
	 */
	public function execute() {
		if ($this->post['add_attribute']) {
			try {
				$this->add_attribute($this->post['new_attribute_name'], $this->post['new_attribute_description']);
				$this->smarty->assign('operation_success', 'Atribut byl přidán');
				$this->display_attributes_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_attributes_list();
			}
		} elseif ($ea=$this->post['edit_attribute']) {//$ea is array with only one element in format: $attributes_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $attributes_id => $v) {
					$this->display_edit_attribute($attributes_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_attribute']) {
			try {
				$this->save_attribute($this->post['attributes_id'], $this->post['name'], $this->post['description']);
				$this->smarty->assign('operation_success', 'Změny byly uloženy.');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=3');
				$this->smarty->display('admin_tab3.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_attribute($this->post['attributes_id']);
			}
		} elseif ($da=$this->post['delete_attribute']) {//$da is array with only one element in format: ? => $attributes_id
			try {
				if(is_array($da)) foreach($da as $attributes_id => $v) {
					$this->delete_attribute($attributes_id);
					$this->smarty->assign('operation_success', 'Atribut byl smazán');
					$this->display_attributes_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());

			}
		} elseif ($this->post['save_attributes_scope']) {
			try {
				$this->save_scope($this->post['propagated_attributes']);
				$this->smarty->assign('operation_success', 'Viditelnost atributů byla aktualizována.');
				$this->display_attributes_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());

			}
		} else {
			$this->display_attributes_list();
		}
	}

	/**
	 * Show list of all attributes
	 *
	 */
	private function display_attributes_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY confirmed"; break;
			case(2): $order_by = "ORDER BY name"; break;
			case(3): $order_by = "ORDER BY description"; break;
			default: $order_by = "";
		}
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attributes` ".$order_by;
		$sql_sec = "
				SELECT attributes_id
				FROM `attrib_storage`
				GROUP BY attributes_id
				HAVING COUNT(attributes_id) > 0";
		$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($attributes)) foreach ($attributes as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$attributes[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_attributes', 1);
		$this->smarty->assign('attributes', $attributes);
		$this->smarty->display('admin_tab3.tpl');
	}

	/**
	 * Display edit form of tha attribute
	 *
	 * @param int $attributes_id
	 */
	private function display_edit_attribute($attributes_id) {
		$sql = "SELECT DISTINCT id, name, description
				FROM `attributes`
				WHERE id='$attributes_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_attribute', 1); //determine which part of template to use
		$this->smarty->assign('attributes_id', $attributes_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('description', $data['description']);
		$this->smarty->display('admin_tab3.tpl');
	}

	/**
	 * Update name and description of any attribute identified by id
	 *
	 * @param int $attributes_id
	 * @param string $new_name
	 * @param string $new_description
	 */
	private function save_attribute($attributes_id, $new_name, $new_description) {
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $new_name))
			throw new Exception('První znak jména vkládaného atributu musí být podtržítko, jméno může obsahovat písmena anglické abecedy a číslice, max. však 32 znaků.');
		if (sizeof($new_description) > 255)
			throw new Exception('Popisek nového atributu je příliš dlouhý');
		if ($this->db->getOne("SELECT id FROM `attributes` WHERE name='$new_name' AND id<>'$attributes_id'"))
			throw new Exception('Atribut se jménem '.$new_name.' již existuje');
		$res =& $this->db->query("UPDATE `attributes` SET name='$new_name', description='$new_description' WHERE id='$attributes_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete attribute
	 *
	 * @param int $attributes_id
	 */
	private function delete_attribute($attributes_id) {
		$this->db->autoCommit(false);
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		$res =& $this->db->query("DELETE FROM `attrib_storage` WHERE attributes_id='$attributes_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}*/
		$res =& $this->db->query("DELETE FROM `attributes` WHERE id='$attributes_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}

	/**
	 * update `attributes` - take all u
	 *
	 * @param array $propagated_attributes  desired public attributes
	 * in format ? => $attributes_id (eg [0] => 1, [1] => 3)
	 */
	private function save_scope($propagated_attributes) {
		//update `attributes` - take all unconfirmed attributes dependnt to the current
		//publication and if there are some, compare their ids to the $propagated_attributes
		$unconfirmed_attributes = $this->db->getAll("SELECT DISTINCT id FROM `attributes` WHERE confirmed=0", array(), DB_FETCHMODE_ORDERED);
		$confirmed_attributes = $this->db->getAll("SELECT DISTINCT id FROM `attributes` WHERE confirmed=1", array(), DB_FETCHMODE_ORDERED);
		$this->db->autoCommit(false);
		//by desire set private attributes as public
		if (is_array($unconfirmed_attributes)) foreach ($unconfirmed_attributes as $id) {
			$id = $id[0];
			if (in_array($id, $propagated_attributes)) {//decide whether administrator wants this attribute to be public or not
				$res =& $this->db->query("UPDATE `attributes` SET confirmed=1 WHERE id='$id' AND confirmed=0");
				if (PEAR::isError($res)) {
					$this->db->rollback();
		    		throw new SAException($res->getMessage());
				}
			}
		}
		//by desire set public attributes as private
		if (is_array($confirmed_attributes)) foreach ($confirmed_attributes as $id) {
			$id = $id[0];
			if (!in_array($id, $propagated_attributes)) {//decide whether administrator wants this attribute to be public or not
				$res =& $this->db->query("UPDATE `attributes` SET confirmed=0 WHERE id='$id' AND confirmed=1");
				if (PEAR::isError($res)) {
					$this->db->rollback();
		    		throw new SAException($res->getMessage());
				}
			}
		}
		$this->db->commit();
	}

	/**
	 * Add new attribute into the database
	 *
	 * @param string $name
	 * @param string $description
	 */
	private function add_attribute($name, $description) {
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $name))
			throw new Exception('První znak jména vkládaného atributu musí být podtržítko, jméno může obsahovat písmena anglické abecedy a číslice, max. však 32 znaků.');
		if (sizeof($description) > 255)
			throw new Exception('Popisek nového atributu je příliš dlouhý');
		if ($this->db->getOne("SELECT id FROM `attributes` WHERE name='$name'"))
			throw new Exception('Atribut se jménem '.$name.' již existuje');
		$submitter_id = $this->auth_user_id;
		$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', 1)";
		$res =& $this->db->query($sql);
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage());
		}
	}
}
?>