<?php
require_once('Admin.inc.php');

/**
 * Managing authors
 *
 */
final class AdminT5 extends Admin {

	/**
	 * Execute admin tab 5
	 *
	 */
	public function execute() {
		if ($this->post['add_author']) {
			try {
				$this->add_author($this->post['name'], $this->post['middlename'], $this->post['surname']);
				$this->smarty->assign('operation_success', 'Autor byl přidán');
				$this->display_authors_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_authors_list();
			}
		} elseif ($ea=$this->post['edit_author']) {//$ea is array with only one element in format: $author_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $author_id => $v) {
					$this->display_edit_author($author_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_author']) {
			try {
				$this->save_author($this->post['author_id'], $this->post['name'], $this->post['middlename'], $this->post['surname']);
				$this->smarty->assign('operation_success', 'Změny byly uloženy.');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=5');
				$this->smarty->display('admin_tab5.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_author($this->post['author_id']);
			}
		} elseif ($da=$this->post['delete_author']) {//$da is array with only one element in format: [...] => $author_id
			try {
				if(is_array($da)) foreach($da as $author_id => $v) {
					$this->delete_author($author_id);
					$this->smarty->assign('operation_success', 'Autor byl smazán');
					$this->display_authors_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} elseif ($this->post['confirm_authors']) {
			try {
				$this->confirm_authors();
				$this->smarty->assign('operation_success', 'Všichni autoři byli schváleni');
				$this->display_authors_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());

			}
		} else {
			$this->display_authors_list();
		}
	}

	/**
	 * Show list of authors
	 *
	 */
	private function display_authors_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY confirmed"; break;
			case(2): $order_by = "ORDER BY name"; break;
			case(3): $order_by = "ORDER BY middlename"; break;
			case(4): $order_by = "ORDER BY surname"; break;
			default: $order_by = '';
		}
		$sql = "SELECT *
				FROM `author` ".$order_by;
		$sql_sec = "
				SELECT author_id
				FROM `author_has_publication`
				GROUP BY author_id
				HAVING COUNT(author_id) > 0";
		$author = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($author)) foreach ($author as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$author[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_author', 1);
		$this->smarty->assign('author', $author);
		$this->smarty->display('admin_tab5.tpl');
	}

	/**
	 * Display edit form of the author
	 *
	 * @param int $author_id
	 */
	private function display_edit_author($author_id) {
		$sql = "SELECT *
				FROM `author`
				WHERE id='$author_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_author', 1); //determine which part of template to use
		$this->smarty->assign('author_id', $author_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('middlename', $data['middlename']);
		$this->smarty->assign('surname', $data['surname']);
		$this->smarty->display('admin_tab5.tpl');
	}

	/**
	 * Validate data sent by web form
	 *
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function validate_author($name, $middlename, $surname) {
		if (sizeof($name) > 20 || !$name) throw new Exception('Délka jména je min. 1 a max. 20 znaků');
		if (sizeof($middlename) > 20) throw new Exception('Délka prostředního jména je max. 20 znaků');
		if (sizeof($surname) > 20 || !$surname) throw new Exception('Délka příjmení je min. 1 a max. 20 znaků');
	}

	/**
	 * Update name and description of any author identified by id
	 *
	 * @param int $author_id
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function save_author($author_id, $name, $middlename, $surname) {
		$this->validate_author($name, $middlename, $surname);
		if ($this->db->getOne("SELECT id FROM `author` WHERE name='$name' AND middlename='$middlename' AND surname='$surname' AND id<>'$author_id'"))
			throw new Exception("autor s tímto jménem již v databázi je");
		$res =& $this->db->query("UPDATE `author` SET name='$name', middlename='$middlename', surname='$surname' WHERE id='$author_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete author
	 *
	 * @param int $author_id
	 */
	private function delete_author($author_id) {
		$this->db->autoCommit(false);
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE author_id='$author_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}*/
		$res =& $this->db->query("DELETE FROM `author` WHERE id='$author_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}

	/**
	 * Set all author as confirmed
	 *
	 */
	private function confirm_authors() {
		$res = $this->db->query("UPDATE `author` SET confirmed=1 WHERE confirmed=0");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
	}

	/**
	 * Add new author into the database
	 *
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function add_author($name, $middlename, $surname) {
		$this->validate_author($name, $middlename, $surname);
		$submitter_id = $this->auth_user_id;
		if ($this->db->getOne("SELECT id FROM `author` WHERE name='$name' AND middlename='$middlename' AND surname='$surname'"))
			throw new Exception("Vkládaný autor již existuje");
		$sql = "INSERT INTO `author` VALUES ('', '$submitter_id', '$name', '$surname', '$middlename', 1)";
		$res =& $this->db->query($sql);
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage());
		}
	}
}
?>