<?php
require_once('Admin.inc.php');

/**
 * Managing users
 *
 */
final class AdminT10 extends Admin
{

	/**
	 * Execute admin tab 10
	 *
	 */
	public function execute()
	{
		if ($ea=$this->post['set_user_level']) {//$ea is array with only one element in format
			try {
				if(is_array($ea)) foreach($ea as $user_id => $v) {
					$this->set_user_level($user_id, $this->post['level'.$user_id]);
					$this->smarty->assign('operation_success', "User has been updated.");
					$this->smarty->assign('refresh', '1 ; URL=index.php?tab=10');
					$this->smarty->display('admin_tab10.tpl');
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_users_list();
			}
		} elseif ($da=$this->post['delete_user']) {//$da is array with only one element
			try {
				if(is_array($da)) foreach($da as $user_id => $v) {
					$this->delete_user($user_id);
					$this->smarty->assign('operation_success', 'User has been deleted.');
					$this->smarty->assign('refresh', '1 ; URL=index.php?tab=10');
					$this->smarty->display('admin_tab10.tpl');
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_users_list();
			}
		} elseif ($da=$this->post['add_user']) {
			try {
				$this->add_user();
				$this->smarty->assign('operation_success', 'User has been added.');
					$this->smarty->assign('refresh', '1 ; URL=index.php?tab=10');
					$this->smarty->display('admin_tab10.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_users_list();
			}
		} else {
			$this->display_users_list();
		}
	}

	/**
	 * Show list of users
	 *
	 */
	private function display_users_list()
	{
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY email"; break;
			default: $order_by = 'ORDER BY name';
		}
		$sql = "SELECT id, name, email
				FROM `submitter` ".$order_by;
		$user = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$this->smarty->assign('user', $user);

		$res = $this->db->query("SELECT id, level FROM `submitter`");
		if (PEAR::isError($res))
    		throw new SAException($res->getMessage());
		while ($row = $res->fetchRow(DB_FETCHMODE_ORDERED))
		{
			$levels[$row[0]] = $row[1];
		}
		$this->smarty->assign('levels', $levels);
		$this->smarty->assign('my_id', $this->auth_module->get_id());
		//feedback add user form
		$this->smarty->assign('login', $_SESSION['f_login']);
		$this->smarty->assign('name', $_SESSION['f_name']);
		$this->smarty->assign('email', $_SESSION['f_email']);
		$this->smarty->assign('pass', $_SESSION['f_pass']);
		$this->smarty->assign('pass_repetition', $_SESSION['f_pass_repetition']);
		$this->smarty->assign('alevel', $_SESSION['f_level']);

		$this->smarty->display('admin_tab10.tpl');
	}

	/**
	 * Add user
	 *
	 */
	private function add_user()
	{
		$name = trim($this->post['name']);
		$_SESSION['f_name'] = $name;
		$email = trim($this->post['email']);
		$_SESSION['f_email'] = $email;
		$login = trim($this->post['login']);
		$_SESSION['f_login'] = $login;
		$pass = trim($this->post['pass']);
		$_SESSION['f_pass'] = $pass;
		$pass_repetition = trim($this->post['pass_repetition']);
		$_SESSION['f_pass_repetition'] = $pass_repetition;
		$level = $this->post['alevel'];
		$_SESSION['f_level'] = $level;

		if (strlen($name) == 0)
			throw new Exception('Name is empty.');
		if (strlen($email) == 0)
			throw new Exception('Email is empty.');
		if (!valid_email($email))
			throw new Exception('Email is invalid.');
		if (strlen($login) == 0)
			throw new Exception('Login is empty.');
		if (strlen($pass) == 0)
			throw new Exception('Password is empty.');
		if (strlen($pass_repetition) == 0)
			throw new Exception('Password repetition is empty.');
		if ($pass != $pass_repetition)
			throw new Exception('Password and password repetition are different.');

		$id = $this->auth_module->create_user($name, $email, $login, $pass, $level);

		unset($_SESSION['f_name']); unset($_SESSION['f_login']); unset($_SESSION['f_email']); unset($_SESSION['f_pass']); unset($_SESSION['f_pass_repetition']); unset($_SESSION['f_level']);
	}


	/**
	 * Sets user permition level for some user
	 *
	 * @param int $user_id
	 * @param int $level
	 */
	private function set_user_level($user_id, $level)
	{
		if (!$this->auth_module->is_admin())
			throw new SAException("Fatal error. You don't have permitions to edit other users.");
		if (!in_array($level, array(0, 1, 2, 3)))
			throw new Exception("Level $level can't be set.");
		$res =& $this->db->query("UPDATE `submitter` SET level='$level' WHERE id='$user_id'");
		if (PEAR::isError($res))
    		throw new SAException($res->getMessage());
	}

	/**
	 * Delete user
	 *
	 * @param int $user_id
	 */
	private function delete_user($user_id)
	{
		if ($user_id == $this->auth_module->get_id())
			throw new SAException("Trying to delete yourself.");
		$res =& $this->db->query("DELETE FROM `submitter` WHERE id='$user_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
	}
}
?>