<?php
require_once('Admin.inc.php');

/**
 * Managing conferences
 *
 */
final class AdminT8 extends Admin
{

	/**
	 * Execute admin tab 8
	 *
	 */
	public function execute() {
		if ($this->post['add_conference']) {
			try {
				add_conference($this->db, $this->post['name'], $this->post['abbreviation'], $this->post['w_year'], $this->post['w_from'], $this->post['w_to'], $this->post['location'], $this->post['isbn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference has been added');
				$this->display_conferences_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_conferences_list();
			}
		} elseif ($ea=$this->post['edit_conference']) {//$ea is array with only one element in format: $conference_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $conference_id => $v) {
					$this->display_edit_conference($conference_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_conference']) {
			try {
				update_conference($this->db, $this->post['conference_id'], $this->post['name'], $this->post['abbreviation'], $this->post['w_year'], $this->post['w_from'], $this->post['w_to'], $this->post['location'], $this->post['isbn'], $this->auth_module->get_id(), true);
				$this->smarty->assign('operation_success', 'The conference has been updated');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=8');
				$this->smarty->display('admin_tab8.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_conference($this->post['conference_id']);
			}
		} elseif ($da=$this->post['delete_conference']) {//$da is array with only one element in format: [...] => $conference_id
			try {
				if(is_array($da)) foreach($da as $conference_id => $v) {
					delete_conference($this->db, $conference_id);
					$this->smarty->assign('operation_success', 'Conference has been deleted');
					$this->display_conferences_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} else {
			$this->display_conferences_list();
		}
	}

	/**
	 * Show list of conferences
	 *
	 */
	private function display_conferences_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY abbreviation"; break;
			case(3): $order_by = "ORDER BY w_from"; break;
			case(4): $order_by = "ORDER BY w_to"; break;
			case(5): $order_by = "ORDER BY w_year"; break;
			case(6): $order_by = "ORDER BY location"; break;
			case(7): $order_by = "ORDER BY isbn"; break;
			default: $order_by = "ORDER BY name";
		}
		$sql = "SELECT *
				FROM `conference`
				WHERE w_year<>'0000' || (w_year='0000' AND submitter_id='".$this->auth_module->get_id()."')
				".$order_by; //select only complete conference with name and year, ommit 'private' non-complete records. Allow displaying non-complete records only to current user
		$sql_sec = "
				SELECT conference_id
				FROM `publication`
				GROUP BY conference_id
				HAVING COUNT(conference_id) > 0";
		$conference = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($conference)) foreach ($conference as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$conference[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_conference', 1);
		$this->smarty->assign('conference', $conference);
		$this->smarty->display('admin_tab8.tpl');
	}

	/**
	 * Display edit form of the conference
	 *
	 * @param int $conference_id
	 */
	private function display_edit_conference($conference_id) {
		$sql = "SELECT *
				FROM `conference`
				WHERE id='$conference_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_conference', 1); //determine which part of template to use
		$this->smarty->assign('conference_id', $conference_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('abbreviation', $data['abbreviation']);
		$this->smarty->assign('w_from', $data['w_from']);
		$this->smarty->assign('w_to', $data['w_to']);
		$this->smarty->assign('w_year', $data['w_year']);
		$this->smarty->assign('location', $data['location']);
		$this->smarty->assign('isbn', $data['isbn']);
		$this->smarty->display('admin_tab8.tpl');
	}
}
?>