<?php
require_once('Admin.inc.php');

final class AdminT9 extends Admin
{

	/**
	 * Execute admin tab 9
	 *
	 */
	public function execute() {
		if ($this->post['add_format']) {
			try {
				$this->add_format($this->post['name'], $this->post['content']);
				$this->smarty->assign('operation_success', 'Template has been added.');
				$this->display_formats_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_formats_list();
			}
		} elseif ($ea=$this->post['edit_format']) {//$ea is array with only one element in format: $format_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $format_id => $v) {
					$this->display_edit_format($format_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($da=$this->post['delete_format']) {//$da is array with only one element in format: [...] => $format_id
			try {
				if(is_array($da)) foreach($da as $format_id => $v) {
					$this->delete_format($format_id);
					$this->smarty->assign('operation_success', 'Template has been deleted.');
					$this->display_formats_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_formats_list();
			}
		} elseif ($this->post['save_format']) {
			try {
				$this->save_format($this->post['format_id'], $this->post['name'], $this->post['content']);
				$this->smarty->assign('operation_success', 'Template has been updated.');
				$this->display_formats_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_format($this->post['format_id']);
			}
		} else {
			$this->display_formats_list();
		}
	}

	/**
	 * Display contents of the format
	 *
	 */
	private function display_formats_list()
	{
		$this->smarty->assign('format', $this->db->getAll('SELECT id, name FROM `format`', DB_FETCHMODE_ASSOC));
		$this->smarty->assign('show_formats', 1);
		$this->smarty->display('admin_tab9.tpl');
	}

	/**
	 * Display edit form of the format
	 *
	 * @param int $format_id
	 */
	private function display_edit_format($format_id)
	{
		$sql = "SELECT name, content
				FROM `format`
				WHERE id='$format_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_format', 1); //determine which part of format to use
		$this->smarty->assign('format_id', $format_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('content', $data['content']);
		$this->smarty->display('admin_tab9.tpl');
	}

	/**
	 * Add new format into the database
	 *
	 * @param string $name
	 * @param string $content  Smarty template
	 */
	private function add_format($name, $content)
	{
		if (sizeof($name) > 40 || !$name)
			throw new Exception('Name of the template should have 1-40 characters.');
		if (!$content)
			throw new Exception('Template definition is missing.');
		if ($this->db->getOne("SELECT id FROM `format` WHERE name='$name'"))
			throw new Exception('Template '.$name.' already exists.');
		$sql = "INSERT INTO `format` VALUES ('', '$name', '$content', ".time().")";
		$res =& $this->db->query($sql);
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete format
	 *
	 * @param int $format_id
	 */
	private function delete_format($format_id)
	{
		if ($this->db->getOne("SELECT COUNT(*) FROM `format`")==1)
			throw new Exception("You can't delete all templates. At leas one must remain in the system.");
		$res =& $this->db->query("DELETE FROM `format` WHERE id='$format_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
	}


	/**
	 * Save contents of format
	 *
	 * @param int $format_id
	 * @param string $name  name of the format
	 * @param string $content  smarty template code
	 */
	private function save_format($format_id, $name, $content)
	{
		if (sizeof($name) > 40 || !$name)
			throw new Exception('Name of the template should have 1-40 characters.');
		if (!$content)
			throw new Exception('Template definition is missing.');
		if ($this->db->getOne("SELECT id FROM `format` WHERE name='$name' AND id<>'$format_id'"))
			throw new Exception('Template '.$name.' already exists.');
		$res =& $this->db->query("UPDATE `format` SET name='$name',
								content='$content',
								timestamp=".time()."  WHERE id='$format_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}
}
?>