<?php
require_once('common/common.inc.php');
require_once('common/db_fns.inc.php');
require_once('common/misc_fns.inc.php');
require_once('libs/smarty/Smarty.class.php');
require_once('common/SException.inc.php');



/**
 * Generator of citations
 *
 */
class CitGenerator
{
	var $db;

	/**
	 * Constructor
	 *
	 * @param PEAR:DB $db Refference to the db object
	 */
	public function __construct(&$db)
	{
		$this->db = $db;
	}

	/**
	 * Returns array with pre-formated citations
	 *
	 * @param $publication_id id of publication
	 * @param int $format_id id of the format
	 * @param array $publ_data associative array of bibtex-style fields
	 * @return textual representaion of the citation
	 */
	public function get_citation($publ_data, $publication_id, $format_id)
	{
		$attributes = get_publication_attributes($this->db, $publication_id);
		return $citation = $this->get_parsed($format_id, $publ_data, $authors, $attributes);
	}

	/**
	 * fetch attributes of the publication, parse it as string, use the propper templates (type1.tpl, type2.tpl, type3.tpl)
	 *
	 * @param int $format_id id of the format for the citation
	 * @param array $column_values associative array extracted from the current row from table `publication`
	 * @param array $authors
	 * @param array $extra_attributes  extra attributes in format array(['name'] => 'value', ...)
	 * @return string  parsed text block
	 */
	private function get_parsed($format_id, $column_values, $authors, $extra_attributes=null)
	{
		//remove html special chars.
		$attributes = htmlspecialchars_rec($attributes);
		$authors = htmlspecialchars_rec($authors);
		$format_id = intval($format_id);
		$extra_attributes = htmlspecialchars_rec($extra_attributes);

		$smarty = new smarty();
		//register the resource name "db", use smarty plug-in functions
		$smarty->register_resource("db", array("db_get_template", "db_get_timestamp", "db_get_secure", "db_get_trusted"));
		//assign all values of the publication to smarty
		foreach ($column_values as $col_name => $value)
		{
			$smarty->assign($col_name, $value);
		}
		$smarty->assign('authors', $authors);
		if(is_array($extra_attributes)) foreach ($extra_attributes as $name => $value)
		{
			$smarty->assign($name, $value);
		}
		return $smarty->fetch("db:$format_id");
	}
}



/*
************ Smarty plugin *************
*/

$db =& db_connect();

function db_get_template($tpl_name, &$tpl_source, &$smarty_obj)
{
	// do database call here to fetch your template,
	// populating $tpl_source
	global $db;
	$res =& $db->query("SELECT content
						FROM `format`
						WHERE id='$tpl_name'");
	if (PEAR::isError($res))
		throw new SException($res->getMessage());
	if ($res->numRows())
	{
		$row = $res->fetchRow();
		$tpl_source = $row[0];
		return true;
	}
	else //else select default template
	{
		$res =& $db->query("SELECT content FROM `format` ORDER BY id");
		if (PEAR::isError($res))
			throw new SException($res->getMessage());
		if ($res->numRows()) {
			$row = $res->fetchRow();
			$tpl_source = $row[0];
			return true;
		}
		else
		{
			return false;
		}
	}
}

function db_get_timestamp($tpl_name, &$tpl_timestamp, &$smarty_obj)
{
	// do database call here to fetch your template,
	// populating $tpl_source
	global $db;
	$res =& $db->query("SELECT timestamp
						FROM `format`
						WHERE id='$tpl_name'");
	if (PEAR::isError($res))
		throw new SException($res->getMessage());
	if ($res->numRows())
	{
		$row = $res->fetchRow();
		$tpl_timestamp = $row[0];
		return true;
	}
	else //else select default template
	{
		$res =& $db->query("SELECT timestamp FROM `format` ORDER BY id");
		if (PEAR::isError($res))
		{
			throw new SException($res->getMessage());
		}
		if ($res->numRows())
		{
			$row = $res->fetchRow();
			$tpl_timestamp = $row[0];
			return true;
		}
		else
		{
			return false;
		}
	}
}

function db_get_secure($tpl_name, &$smarty_obj)
{
	// assume all templates are secure
	return true;
}

function db_get_trusted($tpl_name, &$smarty_obj)
{
	// not used for templates
}


/*
End of Smarty plugin
*/
?>