<?php


/**
 * Generates form as an interface for BibTeX import
 *
 */
class Import
{
	private $db;
	private $smarty;
	private $request;
	private $auth_module;
	private $permitted_extensions;


	/**
	 * Constructor
	 *
	 * @param array $post sanitized post data
	 * @param array $get sanitized get data
	 * @param int $permitted_extensions  permitted file extensions
	 */
	public function __construct($request, $permitted_extensions)
	{
		$this->permitted_extensions = $permitted_extensions;
		$this->request = $request;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);

		$this->smarty = new Smarty;
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
		//if no user logged in, force authorization
		if (!is_numeric($this->auth_module->get_id()))
		{
			$this->smarty->assign('refresh', "1 ; URL=$account_url"."index.php?ref=/import/");
			$this->smarty->assign('operation_error', 'To the import interface you must <a href="'.$account_url.'index.php?ref=/load/">log in</a>.');
			$this->smarty->display('import_login.tpl');
			die();
		}
		if ($this->auth_module->get_level() > 1)
		{
			throw new SIException("Tou don't have rights to import publication.");
		}
	}

	/**
	 * Execute page, entry point of Import class
	 *
	 */
	public function execute()
	{
		$request = $this->request;
		$this->smarty->assign('level', $this->auth_module->get_level());
		if ($request['submit_import'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				if (!$request['bibtex'])
					throw new Exception('Bibtex definiton is missing');
				$this->display_st2();
			} catch (SIException $ex) {
				die($ex->getMessage());
			} catch (Exception $ex) {
				$this->smarty->assign('operation_error', $ex->getMessage());
				$this->display_st1();
			}
		}
		elseif ($request['submit_finish'])
		{
			try {
				$this->validate_files();
				$this->finish();
			} catch (SIException $ex) {
				die($ex->getMessage());
			} catch (Exception $ex) {
				$this->smarty->assign('operation_error', $ex->getMessage());
				$this->display_st2();
			}
		}
		elseif ($request['add_category'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				add_tmp_category($this->db, $request['category_name']);
				$this->smarty->assign('operation_success', 'Category was added');
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['delete_added_categories'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				unset($_SESSION['tmp_cat']);
				$this->smarty->assign("operation_success", "Added categories were deleted");
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else
		{
			$this->feedback_st1();
			$this->display_st1();
		}
	}

	/**
	 * Display the import form
	 *
	 */
	private function display_st1()
	{
		//categories
		$this->feedback_st1();
		$sql = "SELECT id, name
				FROM `categories`
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//added categories
		$s=null;
		if (is_array($_SESSION['tmp_cat'])) foreach($_SESSION['tmp_cat'] as $name)
		{
			$s[] = $name;
		}
		$this->smarty->assign('added_categories', stripslashes_rec(htmlspecialchars_rec($s)));

		$this->smarty->assign('stage', 0);
		$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Try to parse given BibTeX definition, save the result
	 * into $_SESSION['bibtex_parse'] and display the result.
	 *
	 */
	private function display_st2()
	{
		$submitter_id = $this->auth_module->get_id();
		//parsing
		$parser = new Parser($this->request['bibtex']);
		$pub_type = $fields = $authors = null;
		$parser->parse($pub_type, $fields, $authors);
		//save the result
		$_SESSION['bibtex_parse'] = array('pub_type' => $pub_type, 'fields' => $fields, 'authors' => $authors);
		//bibtex validation
		$bibtex = Bibtex::create($pub_type);
		$report = $bibtex->validate($fields);
		$this->smarty->assign('report', $report);
		//bibtex fields
		$this->smarty->assign('pub_type', $pub_type);
		foreach ($fields as $i => $v)
			$this->smarty->assign($i, $v);
		//checked categories
		$s = null;
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories))
			foreach ($propagated_categories as $i => $v) {
				$s[] = $this->db->getOne("SELECT name FROM `categories` WHERE id='$v'");
		}
		$this->smarty->assign('categories', stripslashes_rec(htmlspecialchars_rec($s)));
		//added categories
		$s = null; $tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s[] = $v;
			}
		}
		$this->smarty->assign('tmp_cat', stripslashes_rec(htmlspecialchars_rec($s)));
		$this->smarty->assign('stage', 1);
		$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Finish importing publication, display message about success
	 *
	 */
	private function finish()
	{
		$submitter_id = $this->auth_module->get_id();
		//sanitize input
		$pub_type = addslashes_rec($_SESSION['bibtex_parse']['pub_type']);
		$fields =  addslashes_rec($_SESSION['bibtex_parse']['fields']);
		$authors = addslashes_rec($_SESSION['bibtex_parse']['authors']);

		extract($fields);

		$this->db->autoCommit(false);

		//redistribute conference, journal, publisher and author data into the database
		if ($pub_type == 'inproceedings' || $pub_type == 'proceedings')
		{
			if ($pub_type == 'inproceedings')
			{
				$c_name = $booktitle;
				$booktitle = '';
			}
			else
				$c_name = $title;
			$c_location = $location;
			$location = '';
			$c_year = $year;
			$year = '';
			$c_isbn = $isbn;
			$isbn = '';
			if ($c_name)
			{
				$conference_id = conference_exists($this->db, $c_name, $c_year, $submitter_id);
				if (!$conference_id) //add new conference if it doesn't exist
					$conference_id = add_conference($this->db, $c_name, '', $c_year, '', '', $c_location, $c_isbn, $submitter_id);
			}
		}
		elseif ($pub_type == 'article')
		{
			if ($journal)
			{
				$journal_id = journal_exists($this->db, $journal, $issn);
				if (!$journal_id) //add new journal if it doesn't exist
					$journal_id = add_journal($this->db, $journal, $issn, $submitter_id);
			}
		}
		if ($publisher)
		{
			$publisher_id = publisher_exists($this->db, $publisher);
			if (!$publisher_id) //add new publisher if it doesn't exist
				$publisher_id = add_publisher($this->db, $publisher, $address, $submitter_id);
			$address = '';
		}
		if (count($authors) != 0)
		{
			foreach ($authors as $a)
			{
				$author_id = author_exists($this->db, $a['name'], $a['middlename'], $a['surname']);
				if (!$author_id)
					$author_id = add_author($this->db, $a['name'], $a['middlename'], $a['surname'], $submitter_id);
				$author_ids[] = $author_id; //save author id for further use
			}
		}

		if ($month && !is_numeric($month))
			$issue_date = $year.'-'.strmonth2nummonth($month).'-00';
		else
			$issue_date = $year.'-'.((is_numeric($month) && strlen($month==2))?$month:'00').'-00';

		//insert into `publication`
		$sql = "INSERT INTO `publication`
				(id, submitter_id, publisher_id, conference_id, journal_id, pub_type, title, booktitle, abstract, issue_date, isbn, volume, number, pages, note, chapter, edition, editor, howpublished, institution, organization, school, address, type_of_report, url, confirmed) VALUES
				('', '$submitter_id', '$publisher_id', '$conference_id', '$journal_id', '$pub_type', '$title', '$booktitle', '$abstract', '$issue_date', '$isbn', '$volume', '$number', '$pages', '$note', '$chapter', '$edition', '$editor', '$howpublished', '$institution', '$organization', '$school', '$address', '$type', '$url', 0)";
		$res = $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SLException($res->getMessage());
		}
		$publication_id = last_insert_id($this->db);
		//insert into `author_has_publication`
		if (sizeof($author_ids != 0))
		{
			$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?, ?)");
			if (PEAR::isError($sth))
				throw new SIException($sth->getMessage());
			$priority = 0;
			foreach ($author_ids as $_id)
			{
				$res =& $this->db->execute($sth, array($_id, $publication_id, $priority++));
				if (PEAR::isError($res))
					throw new SIException($res->getMessage());
			}
		}
		//insert into `categories`
		$inserted_cat_ids = null;
		$tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			$sth =& $this->db->prepare("INSERT INTO `categories` VALUES('', '$submitter_id', ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SIException($sth->getMessage());
			}
			foreach ($tmp as $name) {
				$res =& $this->db->execute($sth, stripslashes($name));//stripslashes because prepare-execute add slashes redundantly
				$inserted_cat_ids[] = last_insert_id($this->db);
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}
		//insert into `categories_has_publication`
		$tmp = $_SESSION['tmp_cat'];
		$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SIException($sth->getMessage());
		}
		if (is_array($inserted_cat_ids)) {
			foreach ($inserted_cat_ids as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories)) {
			foreach ($propagated_categories as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}
		//store pdf/ppt into the filesystem
		save_documents($this->db, $publication_id, $title);

		$this->db->commit();

		unset($_SESSION['feedback_st1']);
		unset($_SESSION['bibtex_parse']);

		$this->smarty->assign('operation_success', "Publication has been successfully imported. Now you can modify the publication.");
		$this->smarty->assign('refresh', "1 ; URL=../index.php?pid=$publication_id&amp;tab=2");
		$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Gives user data back to form stage 1
	 *
	 */
	private function feedback_st1()
	{
		$feedback = $_SESSION['feedback_st1'];
		$this->smarty->assign('bibtex', $feedback['bibtex']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
	}

	/**
	 * Validate - check attached files
	 *
	 */
	private function validate_files()
	{
		$permitted_extensions = $this->permitted_extensions;
		foreach($_FILES['documents']["error"] as $key => $error)
		{
			if ($error == UPLOAD_ERR_OK)
			{
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$extension = substr($name, strrpos($name, ".") + 1);
				if ($permitted_extensions)
				{
					if (!in_array($extension, $permitted_extensions))
						throw new Exception("You are adding file with a non-permited extension.");
				}
				if ((!is_uploaded_file($file) && $file))
					throw new SLException('Not an uploaded file.');
			}
		}
	}

	/**
	 * Store given documents
	 *
	 * @param int $publication_id
	 * @param string $title title of the publication
	 * @return string info about document conversion
	 */
	private function save_documents($publication_id, $title)
	{
		$sth = $this->db->prepare("INSERT INTO `documents`
		(`publication_id`, `title`, `content`) VALUES (?, ?, ?)");
		if (PEAR::isError($sth))
			throw new SLException($sth->getMessage());
		$convertor = new Convertor($publication_id, $title, $this->db);
		foreach($_FILES['documents']['error'] as $key => $error)
		{
			if ($error == UPLOAD_ERR_OK)
			{
				if(!file_exists("../storage/$publication_id"))
					mkdir("../storage/$publication_id");
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$dst_filename = realpath("../storage/$publication_id/$name");
				//save document file to filesystem
				$success = move_uploaded_file($file, $dst_filename);
				if (!$success)
					throw new SLException("Can't move uploaded file");
				$convertor->add_file($name);
			}
		}
		return $convertor->finish(); //return info about document conversion
	}

}
?>